@@ -2,6 +2,7 @@
 
 import os
 import os.path
+import struct
 import sys
 import glob
 import http.cookiejar
@@ -10,6 +11,7 @@ import tempfile
 import lz4.block
 import configparser
 import base64
+from io import BytesIO
 from Crypto.Cipher import AES
 from typing import Union
 
@@ -179,7 +181,7 @@ def __expand_win_path(path:Union[dict,str]):
     return os.path.join(os.getenv(path['env'], ''), path['path'])
 
 
-def expand_paths(paths:list, os_name:str):
+def expand_paths_impl(paths:list, os_name:str):
     """Expands user paths on Linux, OSX, and windows"""
 
     os_name = os_name.lower()
@@ -193,8 +195,14 @@ def expand_paths(paths:list, os_name:str):
     else:
         paths = map(os.path.expanduser, paths)
 
-    paths = next(filter(os.path.exists, paths), None)
-    return paths
+    for path in paths:
+        for i in sorted(glob.glob(path)):   # glob will return results in arbitrary order. sorted() is use to make output predictable.
+            yield i                         # can use return here without using `expand_paths()` below.
+                                            # but using generator can be useful if we plan to parse all `Cookies` files later.
+
+
+def expand_paths(paths:list, os_name:str):
+    return next(expand_paths_impl(paths, os_name), None)
 
 
 def text_factory(data):
@@ -394,28 +402,30 @@ class Chrome(ChromiumBased):
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies':[
-                    '~/.config/google-chrome/Default/Cookies',
-                    '~/.config/google-chrome-beta/Default/Cookies'
-                ],
+                '~/.config/google-chrome/Default/Cookies',
+                '~/.config/google-chrome-beta/Default/Cookies'
+            ],
             'windows_cookies':[
-                    {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'}
-                ],
-            'osx_cookies': ['~/Library/Application Support/Google/Chrome/Default/Cookies'],
+                {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Default\\Network\\Cookies'}
+            ],
+            'osx_cookies': [
+                '~/Library/Application Support/Google/Chrome/Default/Cookies',
+                '~/Library/Application Support/Google/Chrome/Profile */Cookies'
+            ],
             'windows_keys': [
-                    {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Local State'}
-                ],
+                {'env':'APPDATA', 'path':'..\\Local\\Google\\Chrome\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'Google\\Chrome\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'Google\\Chrome\\User Data\\Local State'}
+            ],
             'os_crypt_name':'chrome',
             'osx_key_service' : 'Chrome Safe Storage',
             'osx_key_user' : 'Chrome'
         }
-
         super().__init__(browser='Chrome', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 
@@ -425,18 +435,21 @@ class Chromium(ChromiumBased):
         args = {
             'linux_cookies':['~/.config/chromium/Default/Cookies'],
             'windows_cookies':[
-                    {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'}
+                {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Default\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'Chromium\\User Data\\Default\\Network\\Cookies'}
+            ],
+            'osx_cookies': [
+                '~/Library/Application Support/Chromium/Default/Cookies',
+                '~/Library/Application Support/Chromium/Profile */Cookies',
             ],
-            'osx_cookies': ['~/Library/Application Support/Chromium/Default/Cookies'],
             'windows_keys': [
-                    {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'Chromium\\User Data\\Local State'}
+                {'env':'APPDATA', 'path':'..\\Local\\Chromium\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'Chromium\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'Chromium\\User Data\\Local State'}
             ],
             'os_crypt_name':'chromium',
             'osx_key_service' : 'Chromium Safe Storage',
@@ -451,24 +464,23 @@ class Opera(ChromiumBased):
         args = {
             'linux_cookies': ['~/.config/opera/Cookies'],
             'windows_cookies':[
-                    {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
-                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Network\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'},
-                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'}
+                {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
+                {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Network\\Cookies'}
             ],
             'osx_cookies': ['~/Library/Application Support/com.operasoftware.Opera/Cookies'],
             'windows_keys': [
-                    {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Local State'},
-                    {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Local State'}
+                {'env':'APPDATA', 'path':'..\\Local\\Opera Software\\Opera Stable\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'Opera Software\\Opera Stable\\Local State'},
+                {'env':'APPDATA', 'path':'Opera Software\\Opera Stable\\Local State'}
             ],
             'os_crypt_name':'chromium',
             'osx_key_service' : 'Opera Safe Storage',
             'osx_key_user' : 'Opera'
         }
-
         super().__init__(browser='Opera', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 
@@ -476,31 +488,33 @@ class Brave(ChromiumBased):
     def __init__(self, cookie_file=None, domain_name="", key_file=None):
         args = {
             'linux_cookies':[
-                    '~/.config/BraveSoftware/Brave-Browser/Default/Cookies',
-                    '~/.config/BraveSoftware/Brave-Browser-Beta/Default/Cookies'
+                '~/.config/BraveSoftware/Brave-Browser/Default/Cookies',
+                '~/.config/BraveSoftware/Brave-Browser-Beta/Default/Cookies'
             ],
             'windows_cookies':[
-                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Default\\Network\\Cookies'},
             ],
             'osx_cookies': [
-                    '~/Library/Application Support/BraveSoftware/Brave-Browser/Default/Cookies',
-                    '~/Library/Application Support/BraveSoftware/Brave-Browser-Beta/Default/Cookies'
+                '~/Library/Application Support/BraveSoftware/Brave-Browser/Default/Cookies',
+                '~/Library/Application Support/BraveSoftware/Brave-Browser-Beta/Default/Cookies',
+                '~/Library/Application Support/BraveSoftware/Brave-Browser/Profile */Cookies',
+                '~/Library/Application Support/BraveSoftware/Brave-Browser-Beta/Profile */Cookies'
             ],
             'windows_keys': [
-                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browse-Betar\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'}
+                {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'..\\Local\\BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'BraveSoftware\\Brave-Browse-Betar\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'BraveSoftware\\Brave-Browser-Beta\\User Data\\Local State'}
             ],
             'os_crypt_name':'brave',
             'osx_key_service' : 'Brave Safe Storage',
@@ -518,27 +532,60 @@ class Edge(ChromiumBased):
                 '~/.config/microsoft-edge-dev/Default/Cookies'
             ],
             'windows_cookies':[
-                    {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
-                    {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
-                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'}
+                {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Default\\Network\\Cookies'}
+            ],
+            'osx_cookies': [
+                '~/Library/Application Support/Microsoft Edge/Default/Cookies',
+                '~/Library/Application Support/Microsoft Edge/Profile */Cookies'
             ],
-            'osx_cookies': ['~/Library/Application Support/Microsoft Edge/Default/Cookies'],
             'windows_keys': [
-                    {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Local State'},
-                    {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Local State'},
-                    {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Local State'}
+                {'env':'APPDATA', 'path':'..\\Local\\Microsoft\\Edge\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'Microsoft\\Edge\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'Microsoft\\Edge\\User Data\\Local State'}
             ],
             'os_crypt_name':'chromium',
             'osx_key_service' : 'Microsoft Edge Safe Storage',
             'osx_key_user' : 'Microsoft Edge'
         }
-
         super().__init__(browser='Edge', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
 
 
+class Vivaldi(ChromiumBased):
+    """Class for Vivaldi Browser"""
+    def __init__(self, cookie_file=None, domain_name="", key_file=None):
+        args = {
+            'linux_cookies': [
+                '~/.config/vivaldi/Default/Cookies'
+            ],
+            'windows_cookies':[
+                {'env':'APPDATA', 'path':'..\\Local\\Vivaldi\\User Data\\Default\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Vivaldi\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'Vivaldi\\User Data\\Default\\Cookies'},
+                {'env':'APPDATA', 'path':'..\\Local\\Vivaldi\\User Data\\Default\\Network\\Cookies'},
+                {'env':'LOCALAPPDATA', 'path':'Vivaldi\\User Data\\Default\\Network\\Cookies'},
+                {'env':'APPDATA', 'path':'Vivaldi\\User Data\\Default\\Network\\Cookies'}
+            ],
+            'osx_cookies': [
+                '~/Library/Application Support/Vivaldi/Default/Cookies',
+                '~/Library/Application Support/Vivaldi/Profile */Cookies'
+            ],
+            'windows_keys': [
+                {'env':'APPDATA', 'path':'..\\Local\\Vivaldi\\User Data\\Local State'},
+                {'env':'LOCALAPPDATA', 'path':'Vivaldi\\User Data\\Local State'},
+                {'env':'APPDATA', 'path':'Vivaldi\\User Data\\Local State'}
+            ],
+            'os_crypt_name':'chrome',
+            'osx_key_service' : 'Vivaldi Safe Storage',
+            'osx_key_user' : 'Vivaldi'
+        }
+        super().__init__(browser='Vivaldi', cookie_file=cookie_file, domain_name=domain_name, key_file=key_file, **args)
+
+
 class Firefox:
     """Class for Firefox"""
     def __init__(self, cookie_file=None, domain_name=""):
@@ -599,7 +646,11 @@ class Firefox:
             user_data_path = os.path.expanduser(
                 '~/Library/Application Support/Firefox')
         elif sys.platform.startswith('linux'):
-            user_data_path = os.path.expanduser('~/.mozilla/firefox')
+            general_path = os.path.expanduser('~/.mozilla/firefox')
+            if os.path.isdir(general_path):
+                user_data_path = general_path
+            else:
+                user_data_path = os.path.expanduser('~/snap/firefox/common/.mozilla/firefox')
         elif sys.platform == 'win32':
             user_data_path = os.path.join(
                 os.environ.get('APPDATA'), 'Mozilla', 'Firefox')
@@ -672,6 +723,107 @@ class Firefox:
         return cj
 
 
+class Safari:
+    """Class for Safari"""
+
+    APPLE_TO_UNIX_TIME = 978307200
+    NEW_ISSUE_MESSAGE = 'Page format changed.\nPlease create a new issue on: https://github.com/borisbabic/browser_cookie3/issues/new'
+
+    def __init__(self, cookie_file=None, domain_name="") -> None:
+        self.__offset = 0
+        self.__domain_name = domain_name
+        self.__buffer = None
+        self.__open_file(cookie_file)
+        self.__parse_header()
+
+    def __del__(self):
+        if self.__buffer:
+            self.__buffer.close()
+    
+    def __open_file(self, cookie_file):
+        if cookie_file is None:
+            cookie_file = os.path.expanduser('~/Library/Cookies/Cookies.binarycookies')
+        if not os.path.exists(cookie_file):
+            raise BrowserCookieError('Can not find Safari cookie file')
+        self.__buffer = open(cookie_file, 'rb')
+    
+    def __read_file(self, size:int, offset:int=None):
+        if offset is not None:
+            self.__offset = offset
+        self.__buffer.seek(self.__offset)
+        self.__offset += size
+        return BytesIO(self.__buffer.read(size))
+
+    def __parse_header(self):
+        assert self.__buffer.read(4) == b'cook', 'Not a safari cookie file'
+        self.__total_page = struct.unpack('>I', self.__buffer.read(4))[0]
+        
+        self.__page_sizes = []
+        for _ in range(self.__total_page):
+            self.__page_sizes.append(struct.unpack('>I', self.__buffer.read(4))[0])
+
+    @staticmethod
+    def __read_until_null(file:BytesIO, decode:bool=True):
+        data = []
+        while True:
+            byte = file.read(1)
+            if byte == b'\x00':
+                break
+            data.append(byte)
+        data = b''.join(data)
+        if decode:
+            data = data.decode('utf-8')
+        return data
+
+    def __parse_cookie(self, page:BytesIO, cookie_offset:int):
+        page.seek(cookie_offset)
+        cookie_size = struct.unpack('<Q', page.read(8))[0]
+        flags = struct.unpack('<Q', page.read(8))[0]
+        is_secure = bool(flags & 0x1)
+        is_httponly = bool(flags & 0x4)
+        
+        host_offset = struct.unpack('<I', page.read(4))[0]
+        name_offset = struct.unpack('<I', page.read(4))[0]
+        path_offset = struct.unpack('<I', page.read(4))[0]
+        value_offset = struct.unpack('<I', page.read(4))[0]
+
+        assert page.read(8) == b'\x00' * 8, self.NEW_ISSUE_MESSAGE
+        expiry_date = int(struct.unpack('<d', page.read(8))[0] + self.APPLE_TO_UNIX_TIME) # convert to unix time
+        access_time = int(struct.unpack('<d', page.read(8))[0] + self.APPLE_TO_UNIX_TIME) # convert to unix time
+        
+        name = self.__read_until_null(page)
+        value = self.__read_until_null(page)
+        host = self.__read_until_null(page)
+        path = self.__read_until_null(page)
+
+        return create_cookie(host, path, is_secure, expiry_date, name, value, is_httponly)
+
+    def __domain_filter(self, cookie: http.cookiejar.Cookie):
+        if not self.__domain_name:
+            return True
+        return self.__domain_name in cookie.domain
+
+    def __parse_page(self, page_index:int):
+        offset = 8 + self.__total_page * 4 + sum(self.__page_sizes[:page_index])
+        page = self.__read_file(self.__page_sizes[page_index], offset)
+        assert page.read(4) == b'\x00\x00\x01\x00', self.NEW_ISSUE_MESSAGE
+        n_cookies = struct.unpack('<I', page.read(4))[0]
+        cookie_offsets = []
+        for _ in range(n_cookies):
+            cookie_offsets.append(struct.unpack('<I', page.read(4))[0])
+        assert page.read(4) == b'\x00\x00\x00\x00', self.NEW_ISSUE_MESSAGE
+        
+        for offset in cookie_offsets:
+            yield self.__parse_cookie(page, offset)
+    
+    def load(self):
+        cj = http.cookiejar.CookieJar()
+        for i in range(self.__total_page):
+            for cookie in self.__parse_page(i):
+                if self.__domain_filter(cookie):
+                    cj.set_cookie(cookie)
+        return cj
+
 def create_cookie(host, path, secure, expires, name, value, http_only):
     """Shortcut function to create a cookie"""
     # HTTPOnly flag goes in _rest, if present (see https://github.com/python/cpython/pull/17471/files#r511187060)
@@ -715,19 +867,31 @@ def edge(cookie_file=None, domain_name="", key_file=None):
     return Edge(cookie_file, domain_name, key_file).load()
 
 
+def vivaldi(cookie_file=None, domain_name="", key_file=None):
+    """Returns a cookiejar of the cookies used by Vivaldi Browser. Optionally pass in a
+    domain name to only load cookies from the specified domain
+    """
+    return Vivaldi(cookie_file, domain_name, key_file).load()
+
+
 def firefox(cookie_file=None, domain_name=""):
     """Returns a cookiejar of the cookies and sessions used by Firefox. Optionally
     pass in a domain name to only load cookies from the specified domain
     """
     return Firefox(cookie_file, domain_name).load()
 
+def safari(cookie_file=None, domain_name=""):
+    """Returns a cookiejar of the cookies and sessions used by Safari. Optionally
+    pass in a domain name to only load cookies from the specified domain
+    """
+    return Safari(cookie_file, domain_name).load()
 
 def load(domain_name=""):
     """Try to load cookies from all supported browsers and return combined cookiejar
     Optionally pass in a domain name to only load cookies from the specified domain
     """
     cj = http.cookiejar.CookieJar()
-    for cookie_fn in [chrome, chromium, opera, brave, edge, firefox]:
+    for cookie_fn in [chrome, chromium, opera, brave, edge, vivaldi, firefox, safari]:
         try:
             for cookie in cookie_fn(domain_name=domain_name):
                 cj.set_cookie(cookie)
